package view;

import javafx.beans.property.ListProperty;
import javafx.beans.property.Property;
import javafx.beans.property.SimpleListProperty;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.Event;
import javafx.event.EventHandler;
import javafx.event.EventType;
import javafx.fxml.FXML;
import javafx.scene.control.*;
import javafx.scene.text.Text;
import javafx.util.converter.NumberStringConverter;
import modele.Capteur;
import modele.CapteurVirtuel;
import modele.CpuCapteur;

import java.io.IOException;
import java.util.ArrayList;

public class TreeViewCapteur extends FxmlWindow {

    @FXML
    private TreeView<Capteur> tree;
    @FXML
    private TextField name;
    @FXML
    private Text id;
    @FXML
    private TextField temperature;
    private ObservableList<Capteur> lesCapteurs;

    public TreeViewCapteur(ObservableList<Capteur> lesCapteurs, String url, String title) throws IOException {
        super(url, title);
        this.lesCapteurs = lesCapteurs;

        TreeItem<Capteur> root = new TreeItem<>();
        tree.setRoot(root);
        tree.setShowRoot(false);
        for (Capteur capteur : lesCapteurs) {
            TreeItem<Capteur> item = new TreeItem<>(capteur);
            addTreeItem(item);
            root.getChildren().add(item);
        }

    }

    private void addTreeItem(TreeItem<Capteur> cap) {
        if (cap.getValue() instanceof CapteurVirtuel) {
            for (Capteur capteur : ((CapteurVirtuel) cap.getValue()).getLesCapteurs().values()) {
                TreeItem<Capteur> item = new TreeItem<>(capteur);
                addTreeItem(item);
                cap.getChildren().add(item);
            }
        }
    }

    public void initialize() {

        tree.setCellFactory(capteurTreeView -> {
            TreeCell<Capteur> tc = new TreeCell<>() {
                @Override
                protected void updateItem(Capteur item, boolean empty) {
                    super.updateItem(item, empty);
                    if (!empty && item!=null) {
                        textProperty().bindBidirectional(item.nomProperty());
                    } else  {
                        textProperty().unbind();
                        //setText("");
                    }
                }
            };
            return tc;
        });

        tree.getSelectionModel().selectedItemProperty().addListener((observableValue, old, newV) -> {
            if (old != null) {
                name.textProperty().unbindBidirectional(old.getValue().nomProperty());
                id.textProperty().unbindBidirectional(old.getValue().idProperty());
                temperature.textProperty().unbindBidirectional(old.getValue().temperatureProperty());
            }
            if (newV != null) {
                name.textProperty().bindBidirectional(newV.getValue().nomProperty());
                id.textProperty().bindBidirectional(newV.getValue().idProperty());
                temperature.textProperty().bindBidirectional(newV.getValue().temperatureProperty(),new NumberStringConverter());
            }
        });
    }

}
